^ expected errors | v input
// copyright (c) 2017 mike pennisi. all rights reserved.
// this code is governed by the bsd license found in the license file.
/*---
esid: sec-assignment-operators-static-semantics-early-errors
description: Minimal test
---*/

void 0;
