^ expected errors | v input
// This file was procedurally generated from the following sources:
// - foo
// - bar
/*---
esid: sec-assignment-operators-static-semantics-early-errors
description: Applied to a "covered" YieldExpression
flags: [class, generated]
---*/

void 0;
