^ expected errors | v input
// Copyright (C) 2017 Mike Pennisi. All rights reserved.
// See LICENSE or https://github.com/tc39/test262/blob/master/LICENSE
/*---
esid: sec-assignment-operators-static-semantics-early-errors
description: Minimal test
---*/

void 0;
