/* This file was generated by syncqt. */
#ifndef QT_QTQUICK3DRENDER_VERSION_H
#define QT_QTQUICK3DRENDER_VERSION_H

#define QTQUICK3DRENDER_VERSION_STR "5.15.2"

#define QTQUICK3DRENDER_VERSION 0x050F02

#endif // QT_QTQUICK3DRENDER_VERSION_H
